// Part of SimCoupe - A SAM Coup emulator
//
// Serial.cpp: Win32 serial interface
//
//  Copyright (c) 1999-2001  Simon Owen
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

// Notes:
//  The serial interface information in here was extracted from Simon Cooke's
//  terminal emulator "Termite", tho we've not made much use of it yet.

// ToDo:
//  - replace the old quick hack code with a real implementation
//  - more efficient empty data reads, and buffered output?

#include "SimCoupe.h"
#include "Serial.h"


CModemDevice::CModemDevice ()
{
#if 0
    hModem = CreateFile("\\\\.\\COM1", GENERIC_READ|GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);

    COMMTIMEOUTS cto;
    cto.ReadIntervalTimeout = MAXDWORD;
    cto.ReadTotalTimeoutMultiplier = cto.ReadTotalTimeoutConstant = 0;
    cto.WriteTotalTimeoutMultiplier = 50;
    cto.WriteTotalTimeoutConstant = 10;
    SetCommTimeouts(hModem, &cto);
#else
    m_hDevice = INVALID_HANDLE_VALUE;
#endif

    m_nHead = m_nTail = 0;
}

CModemDevice::~CModemDevice ()
{
    if (m_hDevice != INVALID_HANDLE_VALUE)
        CloseHandle(m_hDevice);
}


BYTE CModemDevice::In (WORD wPort_)
{
    // Read more data if we've run out
    if (m_nHead == m_nTail)
    {
        DWORD dwRead = 0;
        ReadFile(m_hDevice, m_abData, sizeof m_abData, &dwRead, NULL);
        m_nHead = 0;

        // No data?
        if (!(m_nTail = dwRead))
            return 0x00;
    }

/*
mr:           EQU  &F8EC ;Mode registers 1 and 2        (RD/WR)
sr:           EQU  &F9EC ;Status register               (RD)
csr:          EQU  &F9EC ;Clock select register         (WR)
cr:           EQU  &FAEC ;Command Register              (WR)

rhr:          EQU  &FBEC ;read holding register         (RD)
thr:          EQU  &FBEC ;transmit holding register     (WR)

acr:          EQU  &FCEC ;Auxilairy control register    (WR)

isr:          EQU  &FDEC ;Interrupt status register     (RD)
imr:          EQU  &FDEC ;Interrupt mask register       (WR)

ctu:          EQU  &FEEC ;counter hi byte               (RD)
ctur:         EQU  &FEEC ;counter hi byte preset        (WR)

ctl:          EQU  &FFEC ;counter lo byte               (RD)
ctlr:         EQU  &FFEC ;counter lo byte preset        (WR)

mr.port:       DEFW &00EC
sr.port:       DEFW &01EC
cr.port:       DEFW &02EC   // bit 4 Z for break
thr.port:      DEFW &03EC
acr.port:      DEFW &04EC
im.port:       DEFW &05EC   // interrupt status: bit 2 NZ for rx, bit 0 NZ for tx
ctu.port:      DEFW &06EC
ctl.port:      DEFW &07EC

mr1.init:      DEFB %11010011   // 211?
mr2.init:      DEFB %00010111   // 23?
csr.init:      DEFB %11001100   // 204?
cint.init:     EQU  %00000101   // 5
*/

    switch (HIBYTE(wPort_))
    {
        case 0x00:
            return 0x00;

        case 0x01:
        {
            BYTE bStatus = 0x08;
            if (m_nHead != m_nTail) bStatus |= 0x01;
            return  bStatus;
        }

        case 0x03:  return m_abData[m_nHead++];

        case 0x05:
            return 0x00;
    }

    return 0x00;
}

void CModemDevice::Out (WORD wPort_, BYTE bVal_)
{
    if (m_hDevice == INVALID_HANDLE_VALUE)
        return;

/*
    port    value written
    ---------------------
    0x02ec, 0x10 (.)    // disable rx & tx, set MR pointer to MR1
    0x00ec, 0x93 ()    // mr1 init
    0x00ec, 0x17 (.)    // mr2 init
    0x01ec, 0x88 ()
    0x04ec, 0x38 (8)    // set power-down/timer mode
    0x05ec, 0x00        // initialise interrupts
    0x06ec, 0x00
    0x07ec, 0x00
    0x02ec, 0x20 ( )
    0x02ec, 0x30 (0)
    0x02ec, 0x40 (@)
    0x02ec, 0x50 (P)

    0x02ec, 0x05 (.)    // 0x5a0 = 1440 = 14400bps?

    0x02ec, 0xa0 ()
*/

    switch (HIBYTE(wPort_))
    {
        case 0x03:
        {
            DWORD dwWritten;
            WriteFile(m_hDevice, &bVal_, 1, &dwWritten, NULL);
            DWORD dwError = GetLastError();
            break;
        }
    }
}
